<?php

namespace Drupal\micro_simplenews\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\Routing\Route;
use Drupal\micro_site\Entity\SiteInterface;
use Drupal\micro_site\SiteUsers;

/**
 * Provides an access checker for site Simplenews edit form.
 */
class SiteSimplenewsAccess {

  /**
   * Checks access to the entity operation on the given route.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route to check against.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The parametrized route.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The currently logged in account.
   * @param \Drupal\micro_site\Entity\SiteInterface $site
   *   The site entity.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(Route $route, RouteMatchInterface $route_match, AccountInterface $account, SiteInterface $site = NULL) {
    if (!$site instanceof SiteInterface) {
      return AccessResult::forbidden('Site entity is NULL.')->addCacheContexts([
        'user',
        'url',
      ]);
    }

    if ($account->hasPermission('administer micro simplenews')) {
      return AccessResult::allowed()->cachePerPermissions()->addCacheContexts([
        'user',
        'url',
      ]);
    }

    if ($account->hasPermission('administer own micro simplenews')) {
      // Site administrators and owner can always manage their menu.
      if ($account->id() == $site->getOwnerId() || in_array($account->id(), $site->getUsersId(SiteUsers::MICRO_SITE_ADMINISTRATOR))) {
        return AccessResult::allowed()->addCacheableDependency($site)->addCacheContexts([
          'user',
          'url',
        ])->cachePerPermissions();
      }
    }

    // No opinion, let's others module give access eventually.
    return AccessResult::neutral();
  }

}
